function [Xn,stretch] = baker(X,al)
%BAKER   Baker's map.
%   XN = BAKER(X) returns the image of X=(x,y) under the action of the
%   baker's map.  BAKER(X,ALPHA) returns the generalized baker's map, where
%   0 < ALPHA < 1.  X must be in the unit square [0,1]^2.  X can also be an
%   array with a 2-vector on each row.
%
%   [XN,STRETCH] = BAKER(X) returns a vector STRETCH that records the
%   vertical stretching experienced by the particle (1/alpha or
%   1/(1-alpha)).  This is used to reconstruct the tangent map.

if nargin < 2
  % Default is the uniform baker's map.
  al = 0.5;
end

if al > 1 || al < 0
  error('Baker''s map requires 0 < alpha < 1.')
end

be = 1-al;
x = X(:,1); y = X(:,2);
xn = zeros(size(x)); yn = zeros(size(y));  % Allocate arrays.

% Formula for y <= alpha.
ila = find(y <= al);
xn(ila) = al*x(ila);
yn(ila) = y(ila)/al;
% Formula for y > alpha.
iga = find(y >  al);
xn(iga) = al + be*x(iga);
yn(iga) = (y(iga) - al)/be;

Xn = [xn yn];

if nargout > 1
  stretch = 1/al*ones(size(xn));
  stretch(iga) = 1/be;
end
